/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.cluster;

import java.io.Serializable;

import org.ow2.orchestra.facade.AccessorUtil;
import org.ow2.orchestra.facade.ManagementAPI;
import org.ow2.orchestra.facade.exception.OrchestraException;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;


/**
 * @author Guillaume Porcher
 *
 */
public class JmxServer implements Server, Serializable {

  private static final long serialVersionUID = -110313016758604653L;

  private String jmxServiceUrl;
  private String jmxObjectName;

  public JmxServer(final String jmxServiceUrl, final String jmxObjectName) {
    this.jmxServiceUrl = jmxServiceUrl;
    this.jmxObjectName = jmxObjectName;
  }

  public JmxServer() {
  }

  /**
   * @return the jmxObjectName
   */
  public String getJmxObjectName() {
    return this.jmxObjectName;
  }

  /**
   * @return the jmxServiceUrl
   */
  public String getJmxServiceUrl() {
    return this.jmxServiceUrl;
  }

  public ManagementAPI getManagementAPI() {
    try {
      return AccessorUtil.getManagementAPI(this.jmxServiceUrl, this.jmxObjectName);
    } catch (final OrchestraException e) {
      throw new OrchestraRuntimeException(e);
    }
  }

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return "[Orchestra JMX node: serviceUrl=" + this.jmxServiceUrl + " objectName=" + this.jmxObjectName + "]";
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj == null || !(obj instanceof JmxServer)) {
      return false;
    }
    final JmxServer other = (JmxServer) obj;
    return this.jmxServiceUrl.equals(other.jmxServiceUrl) && this.jmxObjectName.equals(other.jmxObjectName);
  }

  @Override
  public int hashCode() {
    return this.jmxServiceUrl.hashCode() + this.jmxObjectName.hashCode();
  }
}
