/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.env.binding;

import org.ow2.orchestra.cluster.JmxServer;
import org.ow2.orchestra.pvm.internal.wire.binding.WireDescriptorBinding;
import org.ow2.orchestra.pvm.internal.wire.descriptor.ObjectDescriptor;
import org.ow2.orchestra.pvm.internal.wire.descriptor.StringDescriptor;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Element;

/**
 * parses a descriptor for creating a {@link org.ow2.orchestra.pvm.session.TimerSession}.
 *
 * See schema docs for more details.
 *
 * @author Tom Baeyens, Pascal Verdage
 */
public class JmxServerBinding extends WireDescriptorBinding {

  public JmxServerBinding() {
    super("jmx-server");
  }

  public Object parse(final Element element, final Parse parse, final Parser parser) {
    final ObjectDescriptor objectDescriptor = new ObjectDescriptor();

    final String serviceUrl = XmlUtil.attribute(element, "serviceUrl");
    final String objectName = XmlUtil.attribute(element, "objectName");

    // wire the JobExecutorTimerSession
    objectDescriptor.setClassName(JmxServer.class.getName());

    // inject fields
    objectDescriptor.addInjection("jmxServiceUrl", new StringDescriptor(serviceUrl));
    objectDescriptor.addInjection("jmxObjectName", new StringDescriptor(objectName));
    return objectDescriptor;
  }
}
