/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.env.descriptor;

import java.lang.management.ManagementFactory;
import java.util.Hashtable;
import java.util.Properties;

import javax.management.ObjectName;

import org.hibernate.SessionFactory;
import org.hibernate.jmx.StatisticsService;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.internal.util.Listener;
import org.ow2.orchestra.pvm.internal.wire.WireContext;
import org.ow2.orchestra.pvm.internal.wire.descriptor.HibernateSessionFactoryDescriptor;
import org.ow2.orchestra.util.OrchestraConstants;


/**
 * @author Guillaume Porcher
 *
 */
public class OrchestraHibernateSessionFactoryDescriptor extends HibernateSessionFactoryDescriptor {

  private static final long serialVersionUID = -5414544517550736323L;

  @Override
  public Object construct(final WireContext wireContext) {

    final SessionFactory sessionFactory = (SessionFactory) super.construct(wireContext);

    final Properties orchestraProperties = (Properties) wireContext.get(OrchestraConstants.ORCHESTRA_PROPERTIES);
    if (orchestraProperties != null) {
      final String jmxObjectName = orchestraProperties.getProperty(OrchestraConstants.JMX_OBJECT_NAME);
      final String domainName = jmxObjectName.substring(0, jmxObjectName.indexOf(":"));

      // Construct mbean name from sessionFactoryName
      String mbeanName = this.getName();
      mbeanName = mbeanName.replace(",", "_");
      mbeanName = mbeanName.replace("=", "_");
      mbeanName = mbeanName.replace(":", "_");
      mbeanName = mbeanName.replace("'", "_");
      mbeanName = mbeanName.replace("*", "_");
      mbeanName = mbeanName.replace("?", "_");
      // Register mbean
      final Hashtable<String, String> tb = new Hashtable<String, String>();
      tb.put("type", "Hibernate");
      tb.put("name", mbeanName);
      final ObjectName on;
      try {
        on = new ObjectName(domainName, tb);
        final StatisticsService stats = new StatisticsService();
        stats.setSessionFactory(sessionFactory);
        ManagementFactory.getPlatformMBeanServer().registerMBean(stats, on);
      } catch (final Exception e) {
        throw new OrchestraRuntimeException("couldn't register mbeans", e);
      }

      wireContext.addListener(new SessionFactoryMBeanCloser(on));
    }
    return sessionFactory;
  }

  public static class SessionFactoryMBeanCloser implements Listener {
    private final ObjectName objectName;

    public SessionFactoryMBeanCloser(final ObjectName on) {
      this.objectName = on;
    }

    public void event(final Object source, final String eventName, final Object info) {
      if (WireContext.EVENT_CLOSE.equals(eventName)) {
        try {
          ManagementFactory.getPlatformMBeanServer().unregisterMBean(this.objectName);
        } catch (final Exception e) {
          throw new OrchestraRuntimeException("Exception while unregistering mbean", e);
        }
      }
    }
  }
}
