/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.util.Set;

import org.ow2.orchestra.facade.QuerierRuntimeAPI;
import org.ow2.orchestra.facade.exception.ActivityInstanceNotFoundException;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.OrchestraWrapperException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.jmx.commands.FindActivityInstancesCommand;
import org.ow2.orchestra.jmx.commands.FindProcessInstancesCommand;
import org.ow2.orchestra.jmx.commands.GetActivityInstanceCommand;
import org.ow2.orchestra.jmx.commands.GetProcessInstanceCommand;
import org.ow2.orchestra.pvm.env.EnvironmentFactory;
import org.ow2.orchestra.pvm.internal.cmd.CommandService;

public class QuerierRuntimeAPIImpl implements QuerierRuntimeAPI {

  private final EnvironmentFactory environmentFactory;

  public QuerierRuntimeAPIImpl(final EnvironmentFactory environmentFactory) {
    this.environmentFactory = environmentFactory;
  }


  protected CommandService getCommandService() {
    return this.environmentFactory.get(CommandService.class);
  }

  public Set<ActivityInstance> findActivityInstances(final ProcessInstanceUUID processUUID) throws InstanceNotFoundException {
    try {
      return this.getCommandService().execute(new FindActivityInstancesCommand(processUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof InstanceNotFoundException) {
        throw (InstanceNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public Set<ActivityInstance> findActivityInstances(final ProcessInstanceUUID processUUID, final ActivityState activityState)
    throws InstanceNotFoundException {
    try {
      return this.getCommandService().execute(new FindActivityInstancesCommand(processUUID, activityState));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof InstanceNotFoundException) {
        throw (InstanceNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public Set<ProcessInstance> findProcessInstances() {
    return this.getCommandService().execute(new FindProcessInstancesCommand());
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID)
    throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new FindProcessInstancesCommand(processDefinitionUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityState processInstanceState) throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new FindProcessInstancesCommand(processDefinitionUUID, processInstanceState));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public Set<ProcessInstance> findProcessInstances(final ActivityState processInstanceState) {
    return this.getCommandService().execute(new FindProcessInstancesCommand(processInstanceState));
  }

  public ProcessInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID) throws InstanceNotFoundException {
    try {
      return this.getCommandService().execute(new GetProcessInstanceCommand(processInstanceUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof InstanceNotFoundException) {
        throw (InstanceNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public ActivityInstance getActivityInstance(final ActivityInstanceUUID activityInstanceUUID)
    throws ActivityInstanceNotFoundException {
    try {
      return this.getCommandService().execute(new GetActivityInstanceCommand(activityInstanceUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ActivityInstanceNotFoundException) {
        throw (ActivityInstanceNotFoundException) e.getCause();
      }
      throw e;
    }
  }

}
