/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package org.ow2.orchestra.lang.jaxen;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jaxen.Context;
import org.jaxen.ContextSupport;
import org.jaxen.FunctionContext;
import org.jaxen.JaxenException;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.exception.BpelFaultException.BpelFault;
import org.ow2.orchestra.lang.evaluator.PropertyQueryEvaluator;
import org.w3c.dom.Node;


/**
 * Default evaluator for property queries.
 * @author Alejandro Guizar
 * @version $Revision: 1.1 $ $Date: 2007/09/12 23:20:16 $
 */
class XPathPropertyQueryEvaluator extends XPathEvaluator implements PropertyQueryEvaluator {

  /**
   * The jBPM configuration object name for the resource describing the function library available
   * to property queries.
   */

  private static final long serialVersionUID = 1L;

  private static final Logger LOG = Logger.getLogger(XPathPropertyQueryEvaluator.class.getName());

  private static final FunctionContext FUNCTION_LIBRARY = XPathEvaluator.readFunctionLibrary();

  XPathPropertyQueryEvaluator(final String text) throws JaxenException {
    super(text);
  }

  public Node evaluate(final Node contextNode) {
    try {
      final Context context = this.getContext(contextNode);
      final List nodeSet = this.selectNodes(context);
      return this.narrowToSingleNode(nodeSet);
    } catch (final BpelFaultException e) {
      throw e;
    } catch (final Exception e) {
      XPathPropertyQueryEvaluator.LOG.log(Level.SEVERE, "query evaluation failed", e);
      throw BpelFaultException.generateBpelFaultException(BpelFault.subLanguageExecutionFault, null, e);
    }
  }

  @Override
  protected FunctionContext createFunctionContext() {
    return XPathPropertyQueryEvaluator.FUNCTION_LIBRARY;
  }

  private Context getContext(final Node contextNode) {
    final ContextSupport support = this.getContextSupport();

    final Context context = new Context(support);
    context.setNodeSet(Collections.singletonList(contextNode));
    return context;
  }
}
