/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.parsing.binding;

import java.util.Stack;

import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;

/**
 * @author Gael Lalire
 */
public class DefinitionKeeper {

  private final Stack<Object> parentList;

  private ProcessFullDefinition processFullDefinition;

  private int activityCount;

  private final long processId;

  private ActivityFullDefinition activityFullDefinition;

  public DefinitionKeeper(final long processId) {
    this.processId = processId;
    this.activityCount = 0;
    this.parentList = new Stack<Object>();
  }

  public Object getParent() {
    if (this.parentList.isEmpty()) {
      return null;
    }
    return this.parentList.peek();
  }

  public Stack<Object> getParentsStack() {
    return this.parentList;
  }

  public void pushParent() {
    this.parentList.push(this.activityFullDefinition);
  }

  public void pushParent(final Object parent) {
    this.parentList.push(parent);
  }

  public void popParent() {
    final Object first = this.parentList.pop();
    if (first instanceof BpelActivityFullDefinition) {
      this.activityFullDefinition = (BpelActivityFullDefinition) first;
    }
  }

  public int getNextActivityCount() {
    this.activityCount++;
    return this.activityCount;
  }

  /**
   * @return the processFullDefinition
   */
  public ProcessFullDefinition getProcessFullDefinition() {
    return this.processFullDefinition;
  }

  public void setProcessFullDefinition(final ProcessFullDefinition processFullDefinition) {
    this.processFullDefinition = processFullDefinition;
  }

  /**
   * @return the activityFullDefinition
   */
  public ActivityFullDefinition getActivityFullDefinition() {
    return this.activityFullDefinition;
  }

  /**
   * @param activityFullDefinition the activityFullDefinition to set
   */
  public void setActivityFullDefinition(final ActivityFullDefinition activityFullDefinition) {
    this.activityFullDefinition = activityFullDefinition;
  }


  /**
   * @return the processId
   */
  public long getProcessId() {
    return this.processId;
  }

}
