package org.ow2.orchestra.persistence.db;

import java.util.Date;

import org.ow2.orchestra.facade.data.AbstractData;
import org.ow2.orchestra.facade.data.def.ProcessDefinitionData;
import org.ow2.orchestra.facade.data.runtime.ActivityInstanceData;
import org.ow2.orchestra.facade.data.runtime.BpelFaultData;
import org.ow2.orchestra.facade.data.runtime.CorrelationSetInitializedData;
import org.ow2.orchestra.facade.data.runtime.ForeachInitializedData;
import org.ow2.orchestra.facade.data.runtime.PartnerLinkUpdateData;
import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.data.runtime.ScopeStateUpdateData;
import org.ow2.orchestra.facade.data.runtime.TimerData;
import org.ow2.orchestra.facade.data.runtime.TransitionConditionUpdateData;
import org.ow2.orchestra.facade.data.runtime.VariableUpdateData;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;
import org.ow2.orchestra.services.Recorder;
import org.ow2.orchestra.util.EnvTool;


public class DbRecorder extends DbQuerier implements Recorder {

  public static enum RecordType {
    activityEnded,
    activityExited,
    activityStarted,
    activityTerminated,
    correlationSetInitialized,
    eventStarted,
    exceptionOccured,
    foreachInitialized,
    instanceEnded,
    instanceStarted,
    instanceUpdated,
    partnerLinkUpdated,
    scopeStateChanged,
    timerStarted,
    transitionConditionUpdated,
    variableUpdated
  }

  public static class Record {
    protected long dbid;
    protected ProcessDefinitionUUID processDefinitionUUID;
    protected ProcessInstanceUUID processInstanceUUID;
    protected ActivityInstanceUUID activityInstanceUUID;
    protected AbstractData data;
    protected RecordType type;

    protected Record() { }
    public Record(
        final ProcessDefinitionUUID processDefinitionUUID,
        final ProcessInstanceUUID processInstanceUUID,
        final ActivityInstanceUUID activityInstanceUUID,
        final AbstractData data,
        final RecordType type) {
      super();
      this.processDefinitionUUID = processDefinitionUUID;
      this.processInstanceUUID = processInstanceUUID;
      this.activityInstanceUUID = activityInstanceUUID;
      this.data = data;
      this.type = type;
    }
    public ProcessInstanceUUID getProcessInstanceUUID() {
      return this.processInstanceUUID;
    }
    public ActivityInstanceUUID getActivityInstanceUUID() {
      return this.activityInstanceUUID;
    }

    /**
     * @return the processDefinitionUUID
     */
    public ProcessDefinitionUUID getProcessDefinitionUUID() {
      return this.processDefinitionUUID;
    }
    public RecordType getType() {
      return this.type;
    }
    public AbstractData getData() {
      return this.data;
    }
  }


  public DbRecorder(final QuerierDbSession dbSession) {
    super(dbSession);
  }

  private void storeRecord(final Record record) {
    this.getQuerierDbSession().save(record);
  }

  public void recordActivityEnded(final ActivityInstanceData activityInstanceData) {
    this.storeRecord(new Record(
        activityInstanceData.getProcessInstanceData().getProcessDefinitionData().getUUID(),
        activityInstanceData.getProcessInstanceData().getUUID(),
        activityInstanceData.getUUID(),
        activityInstanceData,
        RecordType.activityEnded));
  }

  public void recordActivityExited(final ActivityInstanceData activityInstanceData) {
    this.storeRecord(new Record(
        activityInstanceData.getProcessInstanceData().getProcessDefinitionData().getUUID(),
        activityInstanceData.getProcessInstanceData().getUUID(),
        activityInstanceData.getUUID(),
        activityInstanceData,
        RecordType.activityExited));
  }

  public void recordActivityStarted(final ActivityInstanceData activityInstanceData) {
    this.storeRecord(new Record(
        activityInstanceData.getProcessInstanceData().getProcessDefinitionData().getUUID(),
        activityInstanceData.getProcessInstanceData().getUUID(),
        activityInstanceData.getUUID(),
        activityInstanceData,
        RecordType.activityStarted));
  }

  public void recordActivityTerminated(final ActivityInstanceData activityInstanceData) {
    this.storeRecord(new Record(
        activityInstanceData.getProcessInstanceData().getProcessDefinitionData().getUUID(),
        activityInstanceData.getProcessInstanceData().getUUID(),
        activityInstanceData.getUUID(),
        activityInstanceData,
        RecordType.activityTerminated));
  }

  public void recordCorrelationSetInitialized(final CorrelationSetInitializedData correlationSetInitializedData) {
    this.storeRecord(new Record(
        null,
        null,
        correlationSetInitializedData.getScopeActivityUUID(),
        correlationSetInitializedData,
        RecordType.correlationSetInitialized));

  }

  public void recordEventStarted(final ActivityInstanceData activityInstanceData) {
    this.storeRecord(new Record(
        activityInstanceData.getProcessInstanceData().getProcessDefinitionData().getUUID(),
        activityInstanceData.getProcessInstanceData().getUUID(),
        activityInstanceData.getUUID(),
        activityInstanceData,
        RecordType.eventStarted));
  }

  public void recordExceptionOccured(final BpelFaultData bpelFaultData) {
    this.storeRecord(new Record(
        null,
        null,
        bpelFaultData.getActivityInstanceUUID(),
        bpelFaultData,
        RecordType.exceptionOccured));

  }

  public void recordForeachInitialized(final ForeachInitializedData foreachInitializedData) {
    this.storeRecord(new Record(
        null,
        null,
        foreachInitializedData.getActivityInstanceUUID(),
        foreachInitializedData,
        RecordType.foreachInitialized));
  }

  public void recordInstanceEnded(final ProcessInstanceData processInstanceData) {
    this.storeRecord(new Record(
        processInstanceData.getProcessDefinitionData().getUUID(),
        processInstanceData.getUUID(),
        processInstanceData.getUUID(),
        processInstanceData,
        RecordType.instanceEnded));
  }

  public void recordInstanceStarted(final ProcessInstanceData processInstanceData) {
    this.storeRecord(new Record(
        processInstanceData.getProcessDefinitionData().getUUID(),
        processInstanceData.getUUID(),
        processInstanceData.getUUID(),
        processInstanceData,
        RecordType.instanceStarted));
  }

  public void recordInstanceUpdated(final ProcessInstanceData processInstanceData) {
    this.storeRecord(new Record(
        processInstanceData.getProcessDefinitionData().getUUID(),
        processInstanceData.getUUID(),
        processInstanceData.getUUID(),
        processInstanceData,
        RecordType.instanceUpdated));

  }

  public void recordPartnerLinkUpdated(final PartnerLinkUpdateData partnerLinkUpdateData) {
    this.storeRecord(new Record(
        null,
        null,
        partnerLinkUpdateData.getScopeActivityUUID(),
        partnerLinkUpdateData,
        RecordType.partnerLinkUpdated));
  }

  public void recordProcessDeployed(final ProcessFullDefinition processFullDefinition) {
    this.getQuerierDbSession().save(processFullDefinition);
  }

  public void recordProcessUndeployed(final ProcessDefinitionUUID processDefinitionUUID) {
    final ProcessFullDefinition processFullDefinition = EnvTool.getJournalQueriers().getProcessDefinition(processDefinitionUUID);
    processFullDefinition.setUndeployedDate(new Date());
  }

  public void recordProcessStateChanged(final ProcessDefinitionData processDefinitionData) {
    final ProcessFullDefinition processFullDefinition =
      EnvTool.getJournalQueriers().getProcessDefinition(processDefinitionData.getUUID());
    processFullDefinition.setState(processDefinitionData.getDate(), processDefinitionData.getState());
  }

  public void recordScopeStateChanged(final ScopeStateUpdateData scopeStateUpdateData) {
    this.storeRecord(new Record(
        null,
        null,
        scopeStateUpdateData.getScopeActivityUUID(),
        scopeStateUpdateData,
        RecordType.scopeStateChanged));
  }

  public void recordTimerStarted(final TimerData timerData) {
    this.storeRecord(new Record(
        null,
        null,
        timerData.getActivityInstanceUUID(),
        timerData,
        RecordType.timerStarted));
  }

  public void recordTransitionConditionUpdated(final TransitionConditionUpdateData transitionConditionUpdateData) {
    this.storeRecord(new Record(
        null,
        null,
        transitionConditionUpdateData.getFlowActivityUUID(),
        transitionConditionUpdateData,
        RecordType.transitionConditionUpdated));
  }

  public void recordVariableUpdated(final VariableUpdateData variableUpdateData) {
    this.storeRecord(new Record(
        null,
        null,
        variableUpdateData.getScopeActivityUUID(),
        variableUpdateData,
        RecordType.variableUpdated));
  }

}
