/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.handlers.impl;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.pvm.internal.jobexecutor.DeadJobHandler;
import org.ow2.orchestra.pvm.job.Job;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.XmlConstants;


/**
 * @author Guillaume Porcher
 *
 */
public class ThrowBpelFaultDeadJobHandler implements DeadJobHandler {

  public static final QName DEAD_JOB_FAULT = new QName(XmlConstants.ORCHESTRA_NS, "jobExecutionFault");

  public void handleDeadJob(final Job deadJob) {
    final BpelExecution jobExecution = (BpelExecution) deadJob.getExecution();
    if (jobExecution != null) {
      jobExecution.handleException(
          jobExecution.getNode(),
          null,
          null,
          new BpelFaultException(ThrowBpelFaultDeadJobHandler.DEAD_JOB_FAULT,
              "job execution failed: " + deadJob.getException()),
          "couldn't execute " + deadJob);
    }
  }

}
