package org.ow2.orchestra.services.itf;

import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.internal.job.JobImpl;
import org.ow2.orchestra.pvm.job.Job;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.WaitingExecution;
import org.ow2.orchestra.runtime.WaitingExecution.WaitingExecutionIterator;
import org.ow2.orchestra.services.ReceivingElement;
import org.w3c.dom.Document;

/**
 * component that stores the process instances.
 *
 * @author Guillaume Porcher
 * @author Charles Souillard
 */
public interface InstanceRepository {

  /**
   *
   * @param process
   *            bpelExecution to store
   */
  void storeInstance(BpelExecution instance);

  void removeInstance(BpelExecution instance);

  void removeInstance(BpelExecution instance, boolean force);

  /**
   * @param child
   */
  void storeExecution(Execution child);
  void removeExecution(Execution child);

  /**
   * Returns one instance.
   */
  BpelExecution getInstance(ProcessInstanceUUID processInstanceUUID);
  /**
   * Returns the list of instances.
   */
  List<BpelExecution> getInstances();

  /**
   * Returns the list of instances for the given BPEL process.
   */
  List<BpelExecution> getInstances(QName processQName);

  /**
   * Returns the list of instances for the given BPEL process.
   */
  List<BpelExecution> getInstances(ProcessDefinitionUUID processUUID);
  /**
   * Returns an up-to-date execution
   *
   * @param old
   *            execution
   */
  BpelExecution getUpdatedExecution(long executionId);

  WaitingExecutionIterator getWaitingExecutions(ReceivingElement receivingElement,
      Map<QName, Document> csProperties);
  WaitingExecution getFirstWaitingExecution(InboundMessageElement inboundMessageElement);

  /**
   * @return the list of jobs that have no retry left.
   */
  List<Job> findJobsWithException();

  /**
   * @param jobId
   * @return the job having this jobId
   */
  JobImpl< ? > getJob(long jobId);
}
