package org.ow2.orchestra.services.itf;

import java.util.Set;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.services.OperationKey;

/**
 * publishes the services for a given
 *
 * @{link BpelProcess}.
 *
 * @author Tom Baeyens
 */
public interface Publisher {

  /** key to use to find an object in the wiring context. */
  String DEFAULT_KEY = "servicePublisher";

  /**
   * Publishes services of the given bpel process (only those that the bpel file
   * can handle (it means those having a corresponding activity in there).
   *
   * @param bpelProcess
   *            bpelProcess to analyse (to deploy every services defined)
   * @param environment
   *            environment
   */
  void publishServices(BpelProcess bpelProcess, Environment environment);

  /**
   * Unpublishes services of the given bpel process (only those that the bpel file
   * can handle (it means those having a corresponding activity in there).
   *
   * @param bpelProcess
   *            bpelProcess to analyse (to undeploy every services defined)
   * @param environment
   *            environment
   */
  void unpublishServices(BpelProcess bpelProcess, Environment environment);

  /**
   * Lists the services of the given bpel process (only those that the bpel file
   * can handle (it means those having a corresponding activity in there).
   *
   * @param bpelProcess
   *            bpelProcess to analyse (to list every services defined)
   * @param environment
   *            environment
   * @return the list of services of the given bpel process
   */
  Set<OperationKey> getServiceOperationKeys(final BpelProcess bpelProcess, final Environment environment);
}
